<?php

/**
 * Plugin Name: Woo AI
 * Description: Using artificial intelligence, it writes descriptions of woocommerce products automatically
 * Text Domain: momoacgwc
 * Domain Path: /languages
 * Author: MoMo Themes
 * Version: 1.2.0
 * Author URI: http://www.momothemes.com/
 * Requires at least: 5.4
 * Tested up to: 6.4.3
 */
/**
 * Freemius SDK Integration
 */

if ( !function_exists( 'momoacgwc_fs' ) ) {
    /**
     * Create a helper function for easy SDK access.
     *
     * @return $momoacgwc_fs
     */
    function momoacgwc_fs()
    {
        global  $momoacgwc_fs ;
        
        if ( !isset( $momoacgwc_fs ) ) {
            // Include Freemius SDK.
            require_once dirname( __FILE__ ) . '/freemius/start.php';
            $momoacgwc_fs = fs_dynamic_init( array(
                'id'             => '15141',
                'slug'           => 'auto-product-description-writer',
                'type'           => 'plugin',
                'public_key'     => 'pk_97c6495005e61f8b0054cb8983a11',
                'is_premium'     => false,
                'premium_suffix' => 'Premium',
                'has_addons'     => false,
                'has_paid_plans' => true,
                'menu'           => array(
                'slug'   => 'momoacgwc-settings',
                'parent' => array(
                'slug' => 'momoacgwc',
            ),
            ),
                'is_live'        => true,
            ) );
        }
        
        return $momoacgwc_fs;
    }
    
    // Init Freemius.
    momoacgwc_fs();
    // Signal that SDK was initiated.
    do_action( 'momoacgwc_fs_loaded' );
}

/**
 * Plugin main class
 */
class MoMo_ACG_For_Woocommerce
{
    /**
     * Plugin Version
     *
     * @var string
     */
    public  $version = '1.2.0' ;
    /**
     * Plugin URL
     *
     * @var string
     */
    public  $plugin_url ;
    /**
     * Plugin Path
     *
     * @var string
     */
    public  $plugin_path ;
    /**
     * Plugin Name
     *
     * @var string
     */
    public  $name ;
    /**
     * Plugin Functions Instance
     *
     * @var MoMo_Basic_Functions_ACG_WC
     */
    public  $fn ;
    /**
     * Plugin Language Instance
     *
     * @var MoMo_ACG_WC_Lang_All
     */
    public  $lang ;
    /**
     * Plugin API Instance
     *
     * @var MoMo_ACG_WC_Rest_API
     */
    public  $api ;
    /**
     * Constructor
     */
    public function __construct()
    {
        add_action( 'plugins_loaded', array( $this, 'momo_acg_wc_plugin_loaded' ) );
        momoacgwc_fs()->add_action( 'after_uninstall', array( $this, 'momoacgwc_fs_uninstall_cleanup' ) );
    }
    
    /**
     * Uninstall
     */
    public function momoacgwc_fs_uninstall_cleanup()
    {
        delete_option( 'momo_acg_wc_edit_product_settings' );
        delete_option( 'momo_acg_wc_openai_settings' );
    }
    
    /**
     * Plugin Loaded
     */
    public function momo_acg_wc_plugin_loaded()
    {
        $this->plugin_url = plugin_dir_url( __FILE__ );
        $this->plugin_path = dirname( __FILE__ ) . '/';
        $this->name = esc_html__( 'Auto Product Description Writer', 'momoacgwc' );
        add_action( 'init', array( $this, 'momo_acg_wc_plugin_init' ), 10 );
    }
    
    /**
     * Plugin Init
     *
     * @return void
     */
    public function momo_acg_wc_plugin_init()
    {
        load_plugin_textdomain( 'momoacgwc', false, 'momo-acg-for-woocommerce/languages' );
        include_once 'includes/class-momo-basic-functions-acg-wc.php';
        include_once 'includes/class-momo-acg-wc-lang-all.php';
        include_once 'includes/class-momo-acg-wc-rest-api.php';
        $this->fn = new MoMo_Basic_Functions_ACG_WC();
        $this->lang = new MoMo_ACG_WC_Lang_All();
        $this->api = new MoMo_ACG_WC_Rest_API();
        
        if ( is_admin() ) {
            include_once 'includes/admin/class-momo-acg-wc-admin-init.php';
            include_once 'includes/admin/class-momo-acg-wc-admin-ajax.php';
            include_once 'includes/admin/metabox/class-momo-acg-wc-metabox.php';
            do_action( 'momo_acg_wc_admin_init' );
        }
    
    }

}
$GLOBALS['momoacgwc'] = new MoMo_ACG_For_Woocommerce();